@@ -56,20 +56,18 @@ public class JAXRSInInterceptor extends AbstractPhaseInterceptor<Message> {
     private static final ResourceBundle BUNDLE = BundleUtils.getBundle(JAXRSInInterceptor.class);
 
     public JAXRSInInterceptor() {
-        super(Phase.PRE_STREAM);
+        super(Phase.UNMARSHAL);
     }
 
     public void handleMessage(Message message) {
         
-        
-        String originalAddress = HttpUtils.getOriginalAddress(message);
-        
         try {
-            processRequest(message, originalAddress);
+            processRequest(message);
         } catch (RuntimeException ex) {
-            Response excResponse = JAXRSUtils.convertFaultToResponse(ex, originalAddress, message);
+            Response excResponse = JAXRSUtils.convertFaultToResponse(ex, message);
             if (excResponse == null) {
-                ProviderFactory.getInstance(originalAddress).clearThreadLocalProxies();
+                ProviderFactory.getInstance(message).clearThreadLocalProxies();
+                message.getExchange().put(Message.PROPOGATE_EXCEPTION, Boolean.TRUE);
                 throw ex;
             }
             message.getExchange().put(Response.class, excResponse);
@@ -78,21 +76,18 @@ public class JAXRSInInterceptor extends AbstractPhaseInterceptor<Message> {
         
     }
     
-    private void processRequest(Message message, String originalAddress) {
+    private void processRequest(Message message) {
         
         if (message.getExchange().get(OperationResourceInfo.class) != null) {
             // it's a suspended invocation;
             return;
         }
         
-        RequestPreprocessor rp = 
-            ProviderFactory.getInstance(originalAddress).getRequestPreprocessor();
+        RequestPreprocessor rp = ProviderFactory.getInstance(message).getRequestPreprocessor();
         if (rp != null) {
             rp.preprocess(message, new UriInfoImpl(message, null));
         }
         
-
-        String httpMethod = (String)message.get(Message.HTTP_REQUEST_METHOD);
         String requestContentType = (String)message.get(Message.CONTENT_TYPE);
         if (requestContentType == null) {
             requestContentType = "*/*";
@@ -121,33 +116,39 @@ public class JAXRSInInterceptor extends AbstractPhaseInterceptor<Message> {
                 new org.apache.cxf.common.i18n.Message("NO_ROOT_EXC", 
                                                    BUNDLE, 
                                                    rawPath);
-            LOG.severe(errorMsg.toString());
+            LOG.warning(errorMsg.toString());
 
             throw new WebApplicationException(Response.Status.NOT_FOUND);
         }
 
         message.getExchange().put(ROOT_RESOURCE_CLASS, resource);
 
+        String httpMethod = (String)message.get(Message.HTTP_REQUEST_METHOD);
         OperationResourceInfo ori = null;     
         
-        List<ProviderInfo<RequestHandler>> shs = 
-            ProviderFactory.getInstance(originalAddress).getRequestHandlers();
+        boolean operChecked = false;
+        List<ProviderInfo<RequestHandler>> shs = ProviderFactory.getInstance(message).getRequestHandlers();
         for (ProviderInfo<RequestHandler> sh : shs) {
             String newAcceptTypes = (String)message.get(Message.ACCEPT_CONTENT_TYPE);
-            if (!acceptTypes.equals(newAcceptTypes) || ori == null) {
+            if (!acceptTypes.equals(newAcceptTypes) || (ori == null && !operChecked)) {
                 acceptTypes = newAcceptTypes;
                 acceptContentTypes = JAXRSUtils.sortMediaTypes(newAcceptTypes);
                 message.getExchange().put(Message.ACCEPT_CONTENT_TYPE, acceptContentTypes);
-                
                 if (ori != null) {
                     values = new MetadataMap<String, String>();
                     resource = JAXRSUtils.selectResourceClass(resources, 
                                                               rawPath, 
                                                               values);
                 }
-                ori = JAXRSUtils.findTargetMethod(resource, values.getFirst(URITemplate.FINAL_MATCH_GROUP), 
-                                                  httpMethod, values, requestContentType, acceptContentTypes);
-                message.getExchange().put(OperationResourceInfo.class, ori);
+                try {                
+                    ori = JAXRSUtils.findTargetMethod(resource, 
+                        values.getFirst(URITemplate.FINAL_MATCH_GROUP), httpMethod, values, 
+                        requestContentType, acceptContentTypes);
+                    message.getExchange().put(OperationResourceInfo.class, ori);
+                } catch (WebApplicationException ex) {
+                    operChecked = true;
+                }
+                
             }
             Response response = sh.getProvider().handleRequest(message, resource);
             if (response != null) {
@@ -167,9 +168,20 @@ public class JAXRSInInterceptor extends AbstractPhaseInterceptor<Message> {
                                                           rawPath, 
                                                           values);
             }
-            ori = JAXRSUtils.findTargetMethod(resource, values.getFirst(URITemplate.FINAL_MATCH_GROUP), 
-                                              httpMethod, values, requestContentType, acceptContentTypes);
-            message.getExchange().put(OperationResourceInfo.class, ori);
+            try {                
+                ori = JAXRSUtils.findTargetMethod(resource, values.getFirst(URITemplate.FINAL_MATCH_GROUP), 
+                                            httpMethod, values, requestContentType, acceptContentTypes);
+                message.getExchange().put(OperationResourceInfo.class, ori);
+            } catch (WebApplicationException ex) {
+                if (ex.getResponse() != null && ex.getResponse().getStatus() == 405 
+                    && "OPTIONS".equalsIgnoreCase(httpMethod)) {
+                    Response response = JAXRSUtils.createResponseBuilder(resource, 200, true).build();
+                    message.getExchange().put(Response.class, response);
+                    return;
+                } else {
+                    throw ex;
+                }
+            }
         }
 
         
