@@ -26,10 +26,9 @@ import java.lang.reflect.Type;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
-import java.util.Map;
+import java.util.logging.Logger;
 
 import javax.ws.rs.core.MediaType;
 import javax.ws.rs.ext.ContextResolver;
@@ -37,95 +36,99 @@ import javax.ws.rs.ext.ExceptionMapper;
 import javax.ws.rs.ext.MessageBodyReader;
 import javax.ws.rs.ext.MessageBodyWriter;
 
-import org.apache.cxf.jaxrs.ext.MappingsHandler;
+import org.apache.cxf.common.logging.LogUtils;
+import org.apache.cxf.endpoint.Endpoint;
+import org.apache.cxf.jaxrs.client.ResponseExceptionMapper;
 import org.apache.cxf.jaxrs.ext.ParameterHandler;
 import org.apache.cxf.jaxrs.ext.RequestHandler;
 import org.apache.cxf.jaxrs.ext.ResponseHandler;
+import org.apache.cxf.jaxrs.ext.SystemQueryHandler;
 import org.apache.cxf.jaxrs.impl.RequestPreprocessor;
 import org.apache.cxf.jaxrs.impl.WebApplicationExceptionMapper;
 import org.apache.cxf.jaxrs.model.ProviderInfo;
+import org.apache.cxf.jaxrs.model.wadl.WadlGenerator;
 import org.apache.cxf.jaxrs.utils.InjectionUtils;
 import org.apache.cxf.jaxrs.utils.JAXRSUtils;
 import org.apache.cxf.message.Message;
 
 public final class ProviderFactory {
+    private static final Logger LOG = LogUtils.getL7dLogger(ProviderFactory.class);
+    private static final ProviderFactory SHARED_FACTORY = new ProviderFactory();
     
-    private static final Map<String, ProviderFactory> FACTORIES = 
-        new HashMap<String, ProviderFactory>();
-    private static final ProviderFactory DEFAULT_FACTORY = new ProviderFactory(); 
-    private static final String SLASH = "/"; 
+    static {
+        // TODO : do dynamic instantiation of JSON and few other default providers
+        JSONProvider jsonProvider = null;
+        try {
+            jsonProvider = new JSONProvider();
+        } catch (Throwable ex) {
+            String message = "Problem with instantiating the default JSON provider, ";
+            if (ex.getMessage() != null) {
+                message += ex.getMessage();
+            } else {
+                message += "exception class : " + ex.getClass().getName();  
+            }
+            LOG.info(message);
+        }
+        
+        SHARED_FACTORY.setProviders(new JAXBElementProvider(),
+                                    jsonProvider,
+                                    new BinaryDataProvider(),
+                                    new SourceProvider(),
+                                    new FormEncodingProvider(),
+                                    new PrimitiveTextProvider(),
+                                    new MultipartProvider(),
+                                    new WebApplicationExceptionMapper(),
+                                    new SystemQueryHandler(),
+                                    new WadlGenerator());
+    }
     
-    private List<ProviderInfo<MessageBodyReader>> defaultMessageReaders = 
+    private List<ProviderInfo<MessageBodyReader>> messageReaders = 
         new ArrayList<ProviderInfo<MessageBodyReader>>();
-    private List<ProviderInfo<MessageBodyWriter>> defaultMessageWriters = 
+    private List<ProviderInfo<MessageBodyWriter>> messageWriters = 
         new ArrayList<ProviderInfo<MessageBodyWriter>>();
-    private List<ProviderInfo<MessageBodyReader>> userMessageReaders = 
-        new ArrayList<ProviderInfo<MessageBodyReader>>(1);
-    private List<ProviderInfo<MessageBodyWriter>> userMessageWriters = 
-        new ArrayList<ProviderInfo<MessageBodyWriter>>(1);
-    private List<ProviderInfo<ContextResolver>> userContextResolvers = 
+    private List<ProviderInfo<ContextResolver>> contextResolvers = 
         new ArrayList<ProviderInfo<ContextResolver>>(1);
-    private List<ProviderInfo<ExceptionMapper>> defaultExceptionMappers = 
-        new ArrayList<ProviderInfo<ExceptionMapper>>(1);
-    private List<ProviderInfo<ExceptionMapper>> userExceptionMappers = 
+    private List<ProviderInfo<ExceptionMapper>> exceptionMappers = 
         new ArrayList<ProviderInfo<ExceptionMapper>>(1);
     private List<ProviderInfo<RequestHandler>> requestHandlers = 
         new ArrayList<ProviderInfo<RequestHandler>>(1);
     private List<ProviderInfo<ResponseHandler>> responseHandlers = 
         new ArrayList<ProviderInfo<ResponseHandler>>(1);
-    private List<ProviderInfo<ParameterHandler>> jaxrsParamHandlers = 
+    private List<ProviderInfo<ParameterHandler>> paramHandlers = 
         new ArrayList<ProviderInfo<ParameterHandler>>(1);
+    private List<ProviderInfo<ResponseExceptionMapper>> responseExceptionMappers = 
+        new ArrayList<ProviderInfo<ResponseExceptionMapper>>(1);
     private RequestPreprocessor requestPreprocessor;
     
     private ProviderFactory() {
-        // TODO : this needs to be done differently,
-        // we need to use cxf-jaxrs-extensions
-        
-        // TODO : make sure the default providers are shared between multiple
-        // factories
-        
-        setProviders(defaultMessageReaders,
-                     defaultMessageWriters,
-                     userContextResolvers,
-                     requestHandlers,
-                     responseHandlers,
-                     defaultExceptionMappers,
-                     jaxrsParamHandlers,
-                     new JAXBElementProvider(),
-                     new JSONProvider(),
-                     new BinaryDataProvider(),
-                     new StringProvider(),
-                     new SourceProvider(),
-                     new FormEncodingReaderProvider(),
-                     new PrimitiveTextProvider(),
-                     new ActivationProvider(),
-                     new WebApplicationExceptionMapper(),
-                     new MappingsHandler());
     }
     
     public static ProviderFactory getInstance() {
-        return getInstance("/");
+        return new ProviderFactory();
     }
     
-    public static ProviderFactory getInstance(String baseAddress) {
-        if (SLASH.equals(baseAddress)) {
-            return DEFAULT_FACTORY;
-        }
-        
-        ProviderFactory pf = null;
-        synchronized (ProviderFactory.class) { 
-            pf = FACTORIES.get(baseAddress);
-            if (pf == null) {
-                pf = new ProviderFactory();
-                FACTORIES.put(baseAddress, pf);
-            }
-        }
-        return pf;
+    public static ProviderFactory getInstance(Message m) {
+        Endpoint e = m.getExchange().get(Endpoint.class);
+        return (ProviderFactory)e.get(ProviderFactory.class.getName());
     }
-
+    
+    public static ProviderFactory getSharedInstance() {
+        return SHARED_FACTORY;
+    }
+    
+    public <T> ContextResolver<T> createContextResolver(Type contextType, 
+                                                        Message m) {
+        Object mt = m.get(Message.CONTENT_TYPE);
+        return createContextResolver(contextType, m,
+               mt == null ? MediaType.valueOf("*/*") : MediaType.valueOf(mt.toString()));
+        
+    }
+    
     @SuppressWarnings("unchecked")
-    public <T> ContextResolver<T> createContextResolver(Type contextType, Message m) {
-        for (ProviderInfo<ContextResolver> cr : userContextResolvers) {
+    public <T> ContextResolver<T> createContextResolver(Type contextType, 
+                                                        Message m,
+                                                        MediaType type) {
+        for (ProviderInfo<ContextResolver> cr : contextResolvers) {
             Type[] types = cr.getProvider().getClass().getGenericInterfaces();
             for (Type t : types) {
                 if (t instanceof ParameterizedType) {
@@ -145,46 +148,30 @@ public final class ProviderFactory {
         return null;
     }
     
-    public <T> ExceptionMapper<T> createExceptionMapper(Class<?> exceptionType, Message m) {
+    
+    public <T extends Throwable> ExceptionMapper<T> createExceptionMapper(Class<?> exceptionType, Message m) {
         
-        ExceptionMapper<T> mapper = doCreateExceptionMapper(userExceptionMappers,
-                                                            exceptionType,
-                                                            m);
-        if (mapper != null) {
+        ExceptionMapper<T> mapper = doCreateExceptionMapper(exceptionType, m);
+        if (mapper != null || this == SHARED_FACTORY) {
             return mapper;
         }
         
-        return doCreateExceptionMapper(defaultExceptionMappers,
-                                       exceptionType,
-                                       m);
+        return SHARED_FACTORY.createExceptionMapper(exceptionType, m);
     }
     
     @SuppressWarnings("unchecked")
-    private static <T> ExceptionMapper<T> doCreateExceptionMapper(
-        List<ProviderInfo<ExceptionMapper>> mappers, Class<?> exceptionType, Message m) {
+    private <T extends Throwable> ExceptionMapper<T> doCreateExceptionMapper(
+        Class<?> exceptionType, Message m) {
         
         List<ExceptionMapper<T>> candidates = new LinkedList<ExceptionMapper<T>>();
         
-        for (ProviderInfo<ExceptionMapper> em : mappers) {
-            Type[] types = em.getProvider().getClass().getGenericInterfaces();
-            for (Type t : types) {
-                if (t instanceof ParameterizedType) {
-                    ParameterizedType pt = (ParameterizedType)t;
-                    Type[] args = pt.getActualTypeArguments();
-                    for (int i = 0; i < args.length; i++) {
-                        if (((Class<?>)args[i]).isAssignableFrom(exceptionType)) {
-                            InjectionUtils.injectContextFields(em.getProvider(), em, m);
-                            InjectionUtils.injectContextMethods(em.getProvider(), em, m);
-                            candidates.add(em.getProvider());
-                        }
-                    }
-                }
-            }
+        for (ProviderInfo<ExceptionMapper> em : exceptionMappers) {
+            handleMapper((List)candidates, em, exceptionType, m);
         }
         if (candidates.size() == 0) {
             return null;
         }
-        Collections.sort((List)candidates, new ExceptionMapperComparator());
+        Collections.sort(candidates, new ExceptionMapperComparator());
         return candidates.get(0);
     }
     
@@ -193,19 +180,8 @@ public final class ProviderFactory {
         
         List<ParameterHandler<T>> candidates = new LinkedList<ParameterHandler<T>>();
         
-        for (ProviderInfo<ParameterHandler> em : jaxrsParamHandlers) {
-            Type[] types = em.getProvider().getClass().getGenericInterfaces();
-            for (Type t : types) {
-                if (t instanceof ParameterizedType) {
-                    ParameterizedType pt = (ParameterizedType)t;
-                    Type[] args = pt.getActualTypeArguments();
-                    for (int i = 0; i < args.length; i++) {
-                        if (((Class<?>)args[i]).isAssignableFrom(paramType)) {
-                            candidates.add(em.getProvider());
-                        }
-                    }
-                }
-            }
+        for (ProviderInfo<ParameterHandler> em : paramHandlers) {
+            handleMapper((List)candidates, em, paramType, null);
         }
         if (candidates.size() == 0) {
             return null;
@@ -214,37 +190,77 @@ public final class ProviderFactory {
         return candidates.get(0);
     }
     
+    @SuppressWarnings("unchecked")
+    public <T extends Throwable> ResponseExceptionMapper<T> createResponseExceptionMapper(
+                                 Class<?> paramType) {
+        
+        List<ResponseExceptionMapper<T>> candidates = new LinkedList<ResponseExceptionMapper<T>>();
+        
+        for (ProviderInfo<ResponseExceptionMapper> em : responseExceptionMappers) {
+            handleMapper((List)candidates, em, paramType, null);
+        }
+        if (candidates.size() == 0) {
+            return null;
+        }
+        Collections.sort(candidates, new ResponseExceptionMapperComparator());
+        return candidates.get(0);
+    }
+    
+    private static void handleMapper(List<Object> candidates, ProviderInfo em, 
+                                     Class<?> expectedType, Message m) {
+        
+        Type[] types = em.getProvider().getClass().getGenericInterfaces();
+        for (Type t : types) {
+            if (t instanceof ParameterizedType) {
+                ParameterizedType pt = (ParameterizedType)t;
+                Type[] args = pt.getActualTypeArguments();
+                for (int i = 0; i < args.length; i++) {
+                    if (((Class<?>)args[i]).isAssignableFrom(expectedType)) {
+                        if (m != null) {
+                            InjectionUtils.injectContextFields(em.getProvider(), em, m);
+                            InjectionUtils.injectContextMethods(em.getProvider(), em, m);
+                        }
+                        candidates.add(em.getProvider());
+                        break;
+                    }
+                }
+            }
+        }
+    }
+    
+    
+    
     public <T> MessageBodyReader<T> createMessageBodyReader(Class<T> bodyType,
                                                             Type parameterType,
                                                             Annotation[] parameterAnnotations,
                                                             MediaType mediaType,
                                                             Message m) {
         // Try user provided providers
-        MessageBodyReader<T> mr = chooseMessageReader(userMessageReaders, 
-                                                      bodyType,
+        MessageBodyReader<T> mr = chooseMessageReader(bodyType,
                                                       parameterType,
                                                       parameterAnnotations,
                                                       mediaType,
                                                       m);
         
         //If none found try the default ones
-        if (mr == null) {
-            mr = chooseMessageReader(defaultMessageReaders,
-                                     bodyType,
-                                     parameterType,
-                                     parameterAnnotations,
-                                     mediaType,
-                                     m);
-        }     
-        
-        return mr;
+        if (mr != null ||  this == SHARED_FACTORY) {
+            return mr;
+        }
+        return SHARED_FACTORY.createMessageBodyReader(bodyType, parameterType, 
+                                                        parameterAnnotations, mediaType, m);
     }
     
     
     
     public List<ProviderInfo<RequestHandler>> getRequestHandlers() {
-        
-        return Collections.unmodifiableList(requestHandlers);
+        if (requestHandlers.size() == 0) {
+            return Collections.unmodifiableList(SHARED_FACTORY.requestHandlers);
+        } else {
+            List<ProviderInfo<RequestHandler>> handlers = 
+                new ArrayList<ProviderInfo<RequestHandler>>(SHARED_FACTORY.requestHandlers);
+            handlers.addAll(requestHandlers);
+            return handlers;
+        }
     }
     
     public List<ProviderInfo<ResponseHandler>> getResponseHandlers() {
@@ -258,59 +274,53 @@ public final class ProviderFactory {
                                                             MediaType mediaType,
                                                             Message m) {
         // Try user provided providers
-        MessageBodyWriter<T> mw = chooseMessageWriter(userMessageWriters,
-                                                      bodyType,
+        MessageBodyWriter<T> mw = chooseMessageWriter(bodyType,
                                                       parameterType,
                                                       parameterAnnotations,
                                                       mediaType,
                                                       m);
         
         //If none found try the default ones
-        if (mw == null) {
-            mw = chooseMessageWriter(defaultMessageWriters,
-                                     bodyType,
-                                     parameterType,
-                                     parameterAnnotations,
-                                     mediaType,
-                                     m);
-        }     
-        
-        return mw;
+        if (mw != null || this == SHARED_FACTORY) {
+            return mw;
+        }
+        return SHARED_FACTORY.createMessageBodyWriter(bodyType, parameterType, 
+                                                        parameterAnnotations, mediaType, m);
     }
     
 //CHECKSTYLE:OFF       
-    private void setProviders(List<ProviderInfo<MessageBodyReader>> readers, 
-                              List<ProviderInfo<MessageBodyWriter>> writers,
-                              List<ProviderInfo<ContextResolver>> resolvers,
-                              List<ProviderInfo<RequestHandler>> requestFilters,
-                              List<ProviderInfo<ResponseHandler>> responseFilters,
-                              List<ProviderInfo<ExceptionMapper>> excMappers,
-                              List<ProviderInfo<ParameterHandler>> paramHandlers,
-                              Object... providers) {
+    private void setProviders(Object... providers) {
         
         for (Object o : providers) {
+            if (o == null) {
+                continue;
+            }
             if (MessageBodyReader.class.isAssignableFrom(o.getClass())) {
-                readers.add(new ProviderInfo<MessageBodyReader>((MessageBodyReader)o)); 
+                messageReaders.add(new ProviderInfo<MessageBodyReader>((MessageBodyReader)o)); 
             }
             
             if (MessageBodyWriter.class.isAssignableFrom(o.getClass())) {
-                writers.add(new ProviderInfo<MessageBodyWriter>((MessageBodyWriter)o)); 
+                messageWriters.add(new ProviderInfo<MessageBodyWriter>((MessageBodyWriter)o)); 
             }
             
             if (ContextResolver.class.isAssignableFrom(o.getClass())) {
-                resolvers.add(new ProviderInfo<ContextResolver>((ContextResolver)o)); 
+                contextResolvers.add(new ProviderInfo<ContextResolver>((ContextResolver)o)); 
             }
             
             if (RequestHandler.class.isAssignableFrom(o.getClass())) {
-                requestFilters.add(new ProviderInfo<RequestHandler>((RequestHandler)o)); 
+                requestHandlers.add(new ProviderInfo<RequestHandler>((RequestHandler)o)); 
             }
             
             if (ResponseHandler.class.isAssignableFrom(o.getClass())) {
-                responseFilters.add(new ProviderInfo<ResponseHandler>((ResponseHandler)o)); 
+                responseHandlers.add(new ProviderInfo<ResponseHandler>((ResponseHandler)o)); 
             }
             
             if (ExceptionMapper.class.isAssignableFrom(o.getClass())) {
-                excMappers.add(new ProviderInfo<ExceptionMapper>((ExceptionMapper)o)); 
+                exceptionMappers.add(new ProviderInfo<ExceptionMapper>((ExceptionMapper)o)); 
+            }
+            
+            if (ResponseExceptionMapper.class.isAssignableFrom(o.getClass())) {
+                responseExceptionMappers.add(new ProviderInfo<ResponseExceptionMapper>((ResponseExceptionMapper)o)); 
             }
             
             if (ParameterHandler.class.isAssignableFrom(o.getClass())) {
@@ -318,10 +328,11 @@ public final class ProviderFactory {
             }
         }
         
-        sortReaders(readers);
-        sortWriters(writers);
+        sortReaders();
+        sortWriters();
         
-        injectContexts(readers, writers, resolvers, requestFilters, responseFilters, excMappers);
+        injectContexts(messageReaders, messageWriters, contextResolvers, requestHandlers, responseHandlers,
+                       exceptionMappers);
     }
 //CHECKSTYLE:ON
     
@@ -341,12 +352,12 @@ public final class ProviderFactory {
      * provider that lists *. Quality parameter values are also used such that
      * x/y;q=1.0 < x/y;q=0.7.
      */    
-    private void sortReaders(List<ProviderInfo<MessageBodyReader>> entityProviders) {
-        Collections.sort(entityProviders, new MessageBodyReaderComparator());
+    private void sortReaders() {
+        Collections.sort(messageReaders, new MessageBodyReaderComparator());
     }
     
-    private void sortWriters(List<ProviderInfo<MessageBodyWriter>> entityProviders) {
-        Collections.sort(entityProviders, new MessageBodyWriterComparator());
+    private void sortWriters() {
+        Collections.sort(messageWriters, new MessageBodyWriterComparator());
     }
     
         
@@ -362,17 +373,16 @@ public final class ProviderFactory {
      * @return
      */
     @SuppressWarnings("unchecked")
-    private <T> MessageBodyReader<T> chooseMessageReader(
-                                 List<ProviderInfo<MessageBodyReader>> readers, 
-                                                         Class<T> type,
+    private <T> MessageBodyReader<T> chooseMessageReader(Class<T> type,
                                                          Type genericType,
                                                          Annotation[] annotations,
                                                          MediaType mediaType,
                                                          Message m) {
-        for (ProviderInfo<MessageBodyReader> ep : readers) {
-            InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
-            InjectionUtils.injectContextMethods(ep.getProvider(), ep, m);             
+        for (ProviderInfo<MessageBodyReader> ep : messageReaders) {
+            
             if (matchesReaderCriterias(ep.getProvider(), type, genericType, annotations, mediaType)) {
+                InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
+                InjectionUtils.injectContextMethods(ep.getProvider(), ep, m);
                 return ep.getProvider();
             }
         }     
@@ -386,16 +396,16 @@ public final class ProviderFactory {
                                                Type genericType,
                                                Annotation[] annotations,
                                                MediaType mediaType) {
-        if (!ep.isReadable(type, genericType, annotations)) {
-            return false;
-        }
-        
         List<MediaType> supportedMediaTypes = JAXRSUtils.getProviderConsumeTypes(ep);
         
         List<MediaType> availableMimeTypes = 
             JAXRSUtils.intersectMimeTypes(Collections.singletonList(mediaType), supportedMediaTypes);
 
-        return availableMimeTypes.size() != 0 ? true : false;
+        if (availableMimeTypes.size() == 0) {
+            return false;
+        }
+        
+        return ep.isReadable(type, genericType, annotations, mediaType);
         
     }
         
@@ -410,17 +420,15 @@ public final class ProviderFactory {
      * @return
      */
     @SuppressWarnings("unchecked")
-    private <T> MessageBodyWriter<T> chooseMessageWriter(
-                          List<ProviderInfo<MessageBodyWriter>> writers, 
-                                                         Class<T> type,
+    private <T> MessageBodyWriter<T> chooseMessageWriter(Class<T> type,
                                                          Type genericType,
                                                          Annotation[] annotations,
                                                          MediaType mediaType,
                                                          Message m) {
-        for (ProviderInfo<MessageBodyWriter> ep : writers) {
-            InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
-            InjectionUtils.injectContextMethods(ep.getProvider(), ep, m); 
+        for (ProviderInfo<MessageBodyWriter> ep : messageWriters) {
             if (matchesWriterCriterias(ep.getProvider(), type, genericType, annotations, mediaType)) {
+                InjectionUtils.injectContextFields(ep.getProvider(), ep, m);
+                InjectionUtils.injectContextMethods(ep.getProvider(), ep, m);
                 return ep.getProvider();
             }
         }     
@@ -434,38 +442,28 @@ public final class ProviderFactory {
                                                Type genericType,
                                                Annotation[] annotations,
                                                MediaType mediaType) {
-        if (!ep.isWriteable(type, genericType, annotations)) {
-            return false;
-        }
-        
         List<MediaType> supportedMediaTypes = JAXRSUtils.getProviderProduceTypes(ep);
         
         List<MediaType> availableMimeTypes = 
             JAXRSUtils.intersectMimeTypes(Collections.singletonList(mediaType),
                                           supportedMediaTypes);
 
-        return availableMimeTypes.size() != 0 ? true : false;
-        
+        if (availableMimeTypes.size() == 0) {
+            return false;
+        }
+        return ep.isWriteable(type, genericType, annotations, mediaType); 
     }
     
-    List<ProviderInfo<MessageBodyReader>> getDefaultMessageReaders() {
-        return Collections.unmodifiableList(defaultMessageReaders);
+    List<ProviderInfo<MessageBodyReader>> getMessageReaders() {
+        return Collections.unmodifiableList(messageReaders);
     }
 
-    List<ProviderInfo<MessageBodyWriter>> getDefaultMessageWriters() {
-        return Collections.unmodifiableList(defaultMessageWriters);
+    List<ProviderInfo<MessageBodyWriter>> getMessageWriters() {
+        return Collections.unmodifiableList(messageWriters);
     }
     
-    List<ProviderInfo<MessageBodyReader>> getUserMessageReaders() {
-        return Collections.unmodifiableList(userMessageReaders);
-    }
-    
-    List<ProviderInfo<MessageBodyWriter>> getUserMessageWriters() {
-        return Collections.unmodifiableList(userMessageWriters);
-    }
-    
-    List<ProviderInfo<ContextResolver>> getUserContextResolvers() {
-        return Collections.unmodifiableList(userContextResolvers);
+    List<ProviderInfo<ContextResolver>> getContextResolvers() {
+        return Collections.unmodifiableList(contextResolvers);
     }
     
      
@@ -477,14 +475,7 @@ public final class ProviderFactory {
      * @param entityProviders the entityProviders to set
      */
     public void setUserProviders(List<?> userProviders) {
-        setProviders(userMessageReaders,
-                     userMessageWriters,
-                     userContextResolvers,
-                     requestHandlers,
-                     responseHandlers,
-                     userExceptionMappers,
-                     jaxrsParamHandlers,
-                     userProviders.toArray());
+        setProviders(userProviders.toArray());
     }
 
     private static class MessageBodyReaderComparator 
@@ -531,14 +522,12 @@ public final class ProviderFactory {
     }
     
     public void clearThreadLocalProxies() {
-        clearProxies(defaultMessageReaders,
-                     defaultMessageWriters,
-                     userMessageReaders,
-                     userMessageWriters,
-                     userContextResolvers,
+        clearProxies(messageReaders,
+                     messageWriters,
+                     contextResolvers,
                      requestHandlers,
                      responseHandlers,
-                     userExceptionMappers);
+                     exceptionMappers);
     }
     
     void clearProxies(List<?> ...lists) {
@@ -551,23 +540,18 @@ public final class ProviderFactory {
     }
     
     void clearProviders() {
-        userMessageReaders.clear();
-        userMessageWriters.clear();
-        userContextResolvers.clear();
-        userExceptionMappers.clear();
+        messageReaders.clear();
+        messageWriters.clear();
+        contextResolvers.clear();
+        exceptionMappers.clear();
         requestHandlers.clear();
         responseHandlers.clear();
-        jaxrsParamHandlers.clear();
+        paramHandlers.clear();
+        responseExceptionMappers.clear();
     }
     
     public void setSchemaLocations(List<String> schemas) {
-        setSchemasOnProviders(userMessageReaders, schemas);
-        setSchemasOnProviders(defaultMessageReaders, schemas);
-    }
-    
-    private void setSchemasOnProviders(List<ProviderInfo<MessageBodyReader>> providers,
-                                       List<String> schemas) {
-        for (ProviderInfo<MessageBodyReader> r : providers) {
+        for (ProviderInfo<MessageBodyReader> r : messageReaders) {
             try {
                 Method m = r.getProvider().getClass().getMethod("setSchemas", 
                                                      new Class[]{List.class});
@@ -588,6 +572,16 @@ public final class ProviderFactory {
         
     }
     
+    private static class ResponseExceptionMapperComparator implements 
+        Comparator<ResponseExceptionMapper<? extends Throwable>> {
+    
+        public int compare(ResponseExceptionMapper<? extends Throwable> em1, 
+                           ResponseExceptionMapper<? extends Throwable> em2) {
+            return compareClasses(em1.getClass(), em2.getClass());
+        }
+        
+    }
+    
     private static class ParameterHandlerComparator implements 
         Comparator<ParameterHandler<? extends Object>> {
 
