@@ -36,6 +36,7 @@ import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -51,14 +52,20 @@ import javax.xml.bind.annotation.XmlMimeType;
 import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;
 import javax.xml.namespace.QName;
 
+import org.w3c.dom.DOMError;
+import org.w3c.dom.DOMErrorHandler;
+
 import org.apache.cxf.BusException;
 import org.apache.cxf.binding.BindingFactoryManager;
 import org.apache.cxf.catalog.CatalogXmlSchemaURIResolver;
+import org.apache.cxf.clustering.FailoverFeature;
+import org.apache.cxf.common.WSDLConstants;
 import org.apache.cxf.common.i18n.Message;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.common.util.StringUtils;
 import org.apache.cxf.common.xmlschema.SchemaCollection;
-import org.apache.cxf.common.xmlschema.XmlSchemaTools;
+import org.apache.cxf.common.xmlschema.XmlSchemaUtils;
+import org.apache.cxf.common.xmlschema.XmlSchemaValidationManager;
 import org.apache.cxf.databinding.DataBinding;
 import org.apache.cxf.databinding.source.mime.MimeAttribute;
 import org.apache.cxf.databinding.source.mime.MimeSerializer;
@@ -94,7 +101,6 @@ import org.apache.cxf.service.model.OperationInfo;
 import org.apache.cxf.service.model.SchemaInfo;
 import org.apache.cxf.service.model.ServiceInfo;
 import org.apache.cxf.service.model.UnwrappedOperationInfo;
-import org.apache.cxf.wsdl.WSDLConstants;
 import org.apache.cxf.wsdl11.WSDLServiceBuilder;
 import org.apache.cxf.wsdl11.WSDLServiceFactory;
 import org.apache.ws.commons.schema.XmlSchema;
@@ -129,6 +135,7 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
     public static final String HEADER = "messagepart.isheader";
     public static final String ELEMENT_NAME = "messagepart.elementName";
     public static final String METHOD = "operation.method";
+    public static final String FORCE_TYPES = "operation.force.types";
     public static final String METHOD_PARAM_ANNOTATIONS = "method.parameters.annotations";
     public static final String METHOD_ANNOTATIONS = "method.return.annotations";
     public static final String PARAM_ANNOTATION = "parameter.annotations";
@@ -155,6 +162,7 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
     private boolean populateFromClass;
     private boolean anonymousWrappers;
     private boolean qualifiedSchemas = true;
+    private boolean validate;
 
     private List<AbstractFeature> features;
     
@@ -163,8 +171,6 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
     private String styleCache;
     private Boolean defWrappedCache;
     
-    
-    
     public ReflectionServiceFactoryBean() {
         getServiceConfigurations().add(0, new DefaultServiceConfiguration());
 
@@ -304,11 +310,22 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
     }
     
     protected void buildServiceFromWSDL(String url) {
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.fine("Creating Service " + getServiceQName() + " from WSDL: " + url);
+        if (LOG.isLoggable(Level.INFO)) {
+            LOG.info("Creating Service " + getServiceQName() + " from WSDL: " + url);
         }
         populateFromClass = false;
         WSDLServiceFactory factory = new WSDLServiceFactory(getBus(), url, getServiceQName());
+        boolean setEPName = true;
+        if (features != null) {
+            for (AbstractFeature f : features) {
+                if (f instanceof FailoverFeature) {
+                    setEPName = false;
+                }
+            }
+        }
+        if (setEPName) {
+            factory.setEndpointName(getEndpointName(false));
+        }
         setService(factory.create());
 
         setServiceProperties();
@@ -325,8 +342,8 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
     }
 
     protected void buildServiceFromClass() {
-        if (LOG.isLoggable(Level.FINE)) {
-            LOG.fine("Creating Service " + getServiceQName() + " from class " + getServiceClass().getName());
+        if (LOG.isLoggable(Level.INFO)) {
+            LOG.info("Creating Service " + getServiceQName() + " from class " + getServiceClass().getName());
         }
         populateFromClass = true;
 
@@ -358,7 +375,7 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
 
         getDataBinding().initialize(service);
 
-        boolean isWrapped = isWrapped();
+        boolean isWrapped = isWrapped() || hasWrappedMethods(serviceInfo.getInterface());
         if (isWrapped) {
             initializeWrappedSchema(serviceInfo);
         }
@@ -387,15 +404,27 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
                 }
             }
         }
-        if (LOG.isLoggable(Level.FINE)) {
+        if (LOG.isLoggable(Level.FINE) || isValidate()) {
             ServiceModelSchemaValidator validator = new ServiceModelSchemaValidator(serviceInfo);
             validator.walk();
             String validationComplaints = validator.getComplaints();
             if (!"".equals(validationComplaints)) {
-                LOG.fine(validationComplaints);
+                if (isValidate()) {
+                    LOG.warning(validationComplaints);
+                } else {
+                    LOG.fine(validationComplaints);
+                }
             }
         }
     }
+    public boolean hasWrappedMethods(InterfaceInfo interfaceInfo) {
+        for (OperationInfo opInfo : interfaceInfo.getOperations()) {
+            if (opInfo.isUnwrappedCapable()) {
+                return true;
+            }
+        }
+        return false;
+    }
 
     protected boolean isFromWsdl() {
         return !populateFromClass && getWsdlURL() != null;
@@ -409,11 +438,21 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
         } else {
             buildServiceFromClass();
         }
-        // validateServiceModel();
+        
+        if (isValidate()) {
+            validateServiceModel();
+        }
     }
 
     public void validateServiceModel() {
+        
+        XmlSchemaValidationManager xsdValidator = getBus().getExtension(XmlSchemaValidationManager.class);
+        
         for (ServiceInfo si : getService().getServiceInfos()) {
+            if (xsdValidator != null) {
+                validateSchemas(xsdValidator, si.getXmlSchemaCollection());
+            }
+            
             for (OperationInfo opInfo : si.getInterface().getOperations()) {
                 for (MessagePartInfo mpi : opInfo.getInput().getMessageParts()) {
                     assert mpi.getXmlSchema() != null;
@@ -469,6 +508,26 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
         }
     }
 
+    private void validateSchemas(XmlSchemaValidationManager xsdValidator, 
+                                 SchemaCollection xmlSchemaCollection) {
+        final boolean[] anyErrors = new boolean[1];
+        final StringBuilder errorBuilder = new StringBuilder();
+        anyErrors[0] = false;
+        xsdValidator.validateSchemas(xmlSchemaCollection.getXmlSchemaCollection(), new DOMErrorHandler() {
+
+            public boolean handleError(DOMError error) {
+                anyErrors[0] = true;
+                errorBuilder.append(error.getMessage());
+                LOG.warning(error.getMessage());
+                return true;
+            }
+        });
+        if (anyErrors[0]) {
+            throw new ServiceConstructionException(new Message("XSD_VALIDATION_ERROR", LOG,
+                                                               errorBuilder.toString()));
+        }
+    }
+
     public boolean isPopulateFromClass() {
         return populateFromClass;
     }
@@ -765,7 +824,8 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
         op.setProperty(METHOD_ANNOTATIONS, m.getAnnotations());
         op.setProperty(METHOD_PARAM_ANNOTATIONS, m.getParameterAnnotations());
 
-        if (!isRPC(m) && isWrapped(m)) {
+        boolean isrpc = isRPC(m);
+        if (!isrpc && isWrapped(m)) {
             UnwrappedOperationInfo uOp = new UnwrappedOperationInfo(op);
             uOp.setProperty(METHOD_ANNOTATIONS, m.getAnnotations());
             uOp.setProperty(METHOD_PARAM_ANNOTATIONS, m.getParameterAnnotations());
@@ -797,6 +857,9 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
                 }
             }
         } else {
+            if (isrpc) {
+                op.setProperty(FORCE_TYPES, Boolean.TRUE);
+            }
             createMessageParts(intf, op, m);
         }
 
@@ -883,9 +946,10 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
         SchemaInfo si = getOrCreateSchema(serviceInfo, mpi.getElementQName().getNamespaceURI(),
                                           getQualifyWrapperSchema());
         XmlSchema schema = si.getSchema();
+        si.setElement(null); //cached element is now invalid
 
         XmlSchemaElement el = new XmlSchemaElement();
-        XmlSchemaTools.setElementQName(el, mpi.getElementQName());
+        XmlSchemaUtils.setElementQName(el, mpi.getElementQName());
         if (!isExistSchemaElement(schema, mpi.getElementQName())) {
             SchemaCollection.addGlobalElementToSchema(schema, el);
         }
@@ -931,7 +995,7 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
         SchemaInfo schemaInfo = getOrCreateSchema(serviceInfo, wrapperBeanName.getNamespaceURI(),
                                                   getQualifyWrapperSchema());
 
-        createWrappedMessageSchema(serviceInfo, wrappedMessage, unwrappedMessage, schemaInfo.getSchema(),
+        createWrappedMessageSchema(serviceInfo, wrappedMessage, unwrappedMessage, schemaInfo,
                                    wrapperBeanName);
     }
 
@@ -991,8 +1055,9 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
                 }
             }
 
+            schemaInfo.setElement(null); //cached element is now invalid
             XmlSchemaElement el = new XmlSchemaElement();
-            XmlSchemaTools.setElementQName(el, qname);
+            XmlSchemaUtils.setElementQName(el, qname);
             el.setNillable(true);
 
             if (!isExistSchemaElement(schema, qname)) {
@@ -1082,11 +1147,13 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
     }
 
     private void createWrappedMessageSchema(ServiceInfo serviceInfo, AbstractMessageContainer wrappedMessage,
-                                            AbstractMessageContainer unwrappedMessage, XmlSchema schema,
+                                            AbstractMessageContainer unwrappedMessage, SchemaInfo info,
                                             QName wrapperName) {
 
+        XmlSchema schema = info.getSchema();
+        info.setElement(null); // the cached schema will be no good
         XmlSchemaElement el = new XmlSchemaElement();
-        XmlSchemaTools.setElementQName(el, wrapperName);
+        XmlSchemaUtils.setElementQName(el, wrapperName);
         SchemaCollection.addGlobalElementToSchema(schema, el);
 
         wrappedMessage.getMessageParts().get(0).setXmlSchema(el);
@@ -1105,12 +1172,12 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
 
         for (MessagePartInfo mpi : unwrappedMessage.getMessageParts()) {
             el = new XmlSchemaElement();
-            XmlSchemaTools.setElementQName(el, mpi.getName());
+            XmlSchemaUtils.setElementQName(el, mpi.getName());
             Map<Class, Boolean> jaxbAnnoMap = getJaxbAnnoMap(mpi);
             if (mpi.isElement()) {
                 addImport(schema, mpi.getElementQName().getNamespaceURI());
-                XmlSchemaTools.setElementQName(el, null);
-                XmlSchemaTools.setElementRefName(el, mpi.getElementQName());
+                XmlSchemaUtils.setElementQName(el, null);
+                XmlSchemaUtils.setElementRefName(el, mpi.getElementQName());
             } else {
                 if (mpi.getTypeQName() != null && !jaxbAnnoMap.containsKey(XmlList.class)) {
                     el.setSchemaTypeName(mpi.getTypeQName());
@@ -1154,7 +1221,7 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
                     mpi.setElement(true);
                     mpi.setElementQName(newName);
                     mpi.setConcreteName(newName); 
-                    XmlSchemaTools.setElementQName(el, newName);
+                    XmlSchemaUtils.setElementQName(el, newName);
                 }
                 
                 if (Collection.class.isAssignableFrom(mpi.getTypeClass())
@@ -1178,7 +1245,7 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
             if (Boolean.TRUE.equals(mpi.getProperty(HEADER))) {
                 QName qn = (QName)mpi.getProperty(ELEMENT_NAME);
 
-                XmlSchemaTools.setElementQName(el, qn);
+                XmlSchemaUtils.setElementQName(el, qn);
 
                 SchemaInfo headerSchemaInfo = getOrCreateSchema(serviceInfo, qn.getNamespaceURI(),
                                                                 getQualifyWrapperSchema());
@@ -1290,8 +1357,7 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
             }
             if (isInParam(method, j)) {
                 final QName q = getInParameterName(op, method, j);
-                final QName q2 = getInPartName(op, method, j);
-                MessagePartInfo part = inMsg.addMessagePart(q2);
+                MessagePartInfo part = inMsg.addMessagePart(getInPartName(op, method, j));
                 initializeParameter(part, paramClasses[j], method.getGenericParameterTypes()[j]);
                 //TODO:remove method param annotations
                 part.setProperty(METHOD_PARAM_ANNOTATIONS, method.getParameterAnnotations());
@@ -1386,11 +1452,40 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
                     }
                 }
             }
-
         }
+        
+        //setting the parameterOrder that
+        //allows preservation of method signatures
+        //when doing java->wsdl->java
+        setParameterOrder(method, paramClasses, op);
 
         initializeFaults(intf, op, method);
     }
+    
+    private void setParameterOrder(Method method, Class[] paramClasses, OperationInfo op) {
+        if (isRPC(method) || !isWrapped(method)) {
+            List<String> paramOrdering = new LinkedList<String>();
+            boolean hasOut = false;
+            for (int j = 0; j < paramClasses.length; j++) {
+                if (Exchange.class.equals(paramClasses[j])) {
+                    continue;
+                }
+                if (isInParam(method, j)) {
+                    paramOrdering.add(getInPartName(op, method, j).getLocalPart());
+                    if (isOutParam(method, j)) {
+                        hasOut = true;
+                    }
+                } else if (isOutParam(method, j)) {
+                    hasOut = true;
+                    paramOrdering.add(getOutPartName(op, method, j).getLocalPart());
+                }
+            }
+            if (!paramOrdering.isEmpty() && hasOut) {
+                op.setParameterOrdering(paramOrdering);
+            }
+        }
+    }
+    
 
     protected void createInputWrappedMessageParts(OperationInfo op, Method method, MessageInfo inMsg) {
         MessagePartInfo part = inMsg.addMessagePart("parameters");
@@ -1446,6 +1541,13 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
                 break;
             }
         }
+        for (MessagePartInfo mpart : op.getOutput().getMessageParts()) {
+            if (Boolean.TRUE.equals(mpart.getProperty(HEADER))) {
+                partName = "result";
+                break;
+            }
+        }
+        
         if (partName == null) {
             partName = "parameters";
         }
@@ -2244,4 +2346,17 @@ public class ReflectionServiceFactoryBean extends AbstractServiceFactoryBean {
     public void setFeatures(List<AbstractFeature> f) {
         this.features = f;
     }
+
+    private boolean isValidate() {
+        return validate || System.getProperty("cxf.validateServiceSchemas", "false").equals("true");
+    }
+
+    /**
+     * If 'validate' is true, this class will validate the service. It will report problems
+     * with the service model and the XML schema for the service.
+     * @param validate
+     */
+    public void setValidate(boolean validate) {
+        this.validate = validate;
+    }
 }
