@@ -23,7 +23,12 @@ import java.io.File;
 import java.io.InputStream;
 import java.net.URL;
 import java.net.URLConnection;
+import java.util.Collection;
+import java.util.List;
 
+import javax.ws.rs.core.Response;
+
+import org.apache.commons.httpclient.Header;
 import org.apache.commons.httpclient.HttpClient;
 import org.apache.commons.httpclient.methods.DeleteMethod;
 import org.apache.commons.httpclient.methods.FileRequestEntity;
@@ -33,7 +38,11 @@ import org.apache.commons.httpclient.methods.PutMethod;
 import org.apache.commons.httpclient.methods.RequestEntity;
 import org.apache.cxf.helpers.IOUtils;
 import org.apache.cxf.io.CachedOutputStream;
+import org.apache.cxf.jaxrs.client.JAXRSClientFactory;
+import org.apache.cxf.jaxrs.client.WebClient;
+import org.apache.cxf.jaxrs.ext.xml.XMLSource;
 import org.apache.cxf.testutil.common.AbstractBusClientServerTestBase;
+
 import org.junit.BeforeClass;
 import org.junit.Test;
 
@@ -45,6 +54,13 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                    launchServer(BookServer.class));
     }
     
+    
+    @Test
+    public void testPropogateException() throws Exception {
+        getAndCompare("http://localhost:9080/bookstore/propogateexception",
+                      "", "application/xml", 500);
+    }
+    
     @Test
     public void testWebApplicationException() throws Exception {
         getAndCompare("http://localhost:9080/bookstore/webappexception",
@@ -52,6 +68,20 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                       "application/xml", 500);
     }
     
+    @Test 
+    public void testAddBookProxyResponse() {
+        BookStore store = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
+        Book b = new Book("CXF rocks", 123L);
+        Response r = store.addBook(b);
+        assertNotNull(r);
+        InputStream is = (InputStream)r.getEntity();
+        assertNotNull(is);
+        XMLSource source = new XMLSource(is);
+        source.setBuffering(true);
+        assertEquals(124L, Long.parseLong(source.getValue("Book/id")));
+        assertEquals("CXF rocks", source.getValue("Book/name"));
+    }
+    
     @Test
     public void testGetBookByURL() throws Exception {
         getAndCompareAsStrings("http://localhost:9080/bookstore/bookurl/http%3A%2F%2Ftest.com%2Frss%2F123",
@@ -59,6 +89,27 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testHeadBookByURL() throws Exception {
+        WebClient wc = 
+            WebClient.create("http://localhost:9080/bookstore/bookurl/http%3A%2F%2Ftest.com%2Frss%2F123");
+        Response response = wc.head();
+        assertTrue(response.getMetadata().size() != 0);
+        assertEquals(0, ((InputStream)response.getEntity()).available());
+    }
+    
+    @Test
+    public void testOptions() throws Exception {
+        WebClient wc = 
+            WebClient.create("http://localhost:9080/bookstore/bookurl/http%3A%2F%2Ftest.com%2Frss%2F123");
+        Response response = wc.options();
+        List<Object> values = response.getMetadata().get("Allow");
+        assertNotNull(values);
+        assertTrue(values.contains("POST") && values.contains("GET")
+                   && values.contains("DELETE") && values.contains("PUT"));
+        assertEquals(0, ((InputStream)response.getEntity()).available());
+    }
+    
     @Test
     public void testGetBookByEncodedQuery() throws Exception {
         getAndCompareAsStrings("http://localhost:9080/bookstore/bookquery?"
@@ -67,7 +118,28 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testGetGenericBook() throws Exception {
+        getAndCompareAsStrings("http://localhost:9080/bookstore/genericbooks/123",
+                               "resources/expected_get_book123.txt",
+                               "application/xml", 200);
+    }
     
+    @Test
+    public void testGetGenericResponseBook() throws Exception {
+        getAndCompareAsStrings("http://localhost:9080/bookstore/genericresponse/123",
+                               "resources/expected_get_book123.txt",
+                               "application/xml", 200);
+    }
+    
+    @Test
+    public void testGetBookByArrayQuery() throws Exception {
+        getAndCompareAsStrings("http://localhost:9080/bookstore/bookidarray?"
+                               + "id=1&id=2&id=3",
+                               "resources/expected_get_book123.txt",
+                               "application/xml", 200);
+    }
+        
     @Test
     public void testNoRootResourceException() throws Exception {
         getAndCompare("http://localhost:9080/nobookstore/webappexception",
@@ -82,6 +154,19 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                       "application/xml", 404);
     }
     
+    @Test
+    public void testWriteAndFailEarly() throws Exception {
+        getAndCompare("http://localhost:9080/bookstore/books/fail-early",
+                      "This is supposed to go on the wire",
+                      "application/bar, text/plain", 410);
+    }
+    
+    @Test
+    public void testWriteAndFailLate() throws Exception {
+        getAndCompare("http://localhost:9080/bookstore/books/fail-late",
+                      "", "application/bar", 410);
+    }
+    
     
     @Test
     public void testAcceptTypeMismatch() throws Exception {
@@ -121,13 +206,11 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     
     @Test
     public void testNoMessageWriterFound() throws Exception {
-        // TODO : more specific message is needed
         String msg1 = ".No message body writer found for response class : GregorianCalendar.";
         String msg2 = ".No message body writer found for response class : Calendar.";
         
-        getAndCompareStrings("http://localhost:9080/bookstore/timetable",
-                      msg1, msg2,
-                      "*/*", 500);
+        getAndCompareStrings("http://localhost:9080/bookstore/timetable", 
+                             new String[]{msg1, msg2}, "*/*", 500);
     }
     
     @SuppressWarnings("deprecation")
@@ -174,7 +257,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     @Test
     public void testBookExists() throws Exception {
         checkBook("http://localhost:9080/bookstore/books/check/123", true);
-        checkBook("http://localhost:9080/bookstore/books/check/124", false);  
+        checkBook("http://localhost:9080/bookstore/books/check/125", false);  
         
     }
     
@@ -206,6 +289,48 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                "application/xml", 200);
     }
     
+    @Test
+    public void testGetBook123WebClient() throws Exception {
+        BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
+        Book b = bs.getBook("123");
+        assertEquals(b.getId(), 123);
+    }
+    
+    @Test
+    public void testDeleteWithWebClient() throws Exception {
+        BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
+        Response r = bs.deleteBook("123");
+        assertEquals(200, r.getStatus());
+    }
+    
+    @Test
+    public void testCreatePut() throws Exception {
+        BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
+        Response r = bs.createBook(777L);
+        assertEquals(200, r.getStatus());
+    }
+    
+    @Test
+    public void testUpdateWithWebClient() throws Exception {
+        BookStore bs = JAXRSClientFactory.create("http://localhost:9080", BookStore.class);
+        Book book = new Book();
+        book.setId(888);
+        bs.updateBook(book);
+        assertEquals(304, WebClient.client(bs).getResponse().getStatus());
+    }
+    
+    @Test
+    public void testGetBookTypeAndWildcard() throws Exception {
+        getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
+                               "resources/expected_get_book123.txt",
+                               "application/xml;q=0.8,*/*", 
+                               "application/xml", 200);
+        getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
+                               "resources/expected_get_book123.txt",
+                               "application/*", 
+                               "application/xml", 200);
+    }
+    
     @Test
     public void testGetBook123() throws Exception {
         getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
@@ -226,9 +351,31 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         
         getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
                                "resources/expected_get_book123json.txt",
-                               "application/xml,application/json", 200);
+                               "application/json, application/xml", 
+                               "application/json", 200);
+        
+        getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
+                               "resources/expected_get_book123.txt",
+                               "application/xml, application/json", 
+                               "application/xml", 200);
     }
     
+    @Test
+    public void testGetBookXmlWildcard() throws Exception {
+        
+        getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
+                               "resources/expected_get_book123.txt",
+                               "*/*", "application/xml", 200);
+        
+    }
+    
+    @Test
+    public void testGetBookBuffer() throws Exception {
+        getAndCompareAsStrings("http://localhost:9080/bookstore/books/buffer",
+                               "resources/expected_get_book123.txt",
+                               "application/bar", 200);
+    }    
+    
     @Test
     public void testGetBookBySegment() throws Exception {
         getAndCompareAsStrings("http://localhost:9080/bookstore/segment/matrix2;first=12;second=3",
@@ -262,7 +409,21 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     public void testGetBookByHeader() throws Exception {
         getAndCompareAsStrings("http://localhost:9080/bookstore/bookheaders",
                                "resources/expected_get_book123.txt",
-                               "application/xml", 200);
+                               "application/xml;q=0.5,text/xml", "text/xml", 200);
+    }
+    
+    @Test
+    public void testGetBookByHeaderPerRequest() throws Exception {
+        getAndCompareAsStrings("http://localhost:9080/bookstore2/bookheaders",
+                               "resources/expected_get_book123.txt",
+                               "application/xml;q=0.5,text/xml", "text/xml", 200);
+    }
+    
+    @Test
+    public void testGetBookByHeaderDefault() throws Exception {
+        getAndCompareAsStrings("http://localhost:9080/bookstore/bookheaders2",
+                               "resources/expected_get_book123.txt",
+                               "application/xml;q=0.5,text/xml", "text/xml", 200);
     }
     
     @Test
@@ -287,14 +448,15 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         
         getAndCompareAsStrings("http://localhost:9080/bookstore/books/123",
                                "resources/expected_get_book123json.txt",
-                               "application/xml,application/json", 200);
+                               "application/xml;q=0.1,application/json", "application/json", 200);
     }
     
     @Test
     public void testGetBook123FromSubObject() throws Exception {
         getAndCompareAsStrings(
             "http://localhost:9080/bookstore/booksubresourceobject/123/chaptersobject/sub/1",
-            "resources/expected_get_chapter1.txt", "application/xml", 200);
+            "resources/expected_get_chapter1.txt", "application/xml",
+            "application/xml;charset=iso-8859-1", 200);
     }
     
     @Test
@@ -302,7 +464,15 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         
         getAndCompareAsStrings("http://localhost:9080/bookstore/booksubresource/123/chapters/1",
                                "resources/expected_get_chapter1.txt",
-                               "application/xml", 200);
+                               "application/xml", "application/xml;charset=iso-8859-1", 200);
+    }
+    
+    @Test
+    public void testGetChapterEncodingDefault() throws Exception {
+        
+        getAndCompareAsStrings("http://localhost:9080/bookstore/booksubresource/123/chapters/badencoding/1",
+                               "resources/expected_get_chapter1_utf.txt",
+                               "application/xml", "application/xml;charset=UTF-8", 200);
     }
     
     @Test
@@ -311,6 +481,9 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         getAndCompareAsStrings("http://localhost:9080/bookstore/booksubresource/123/chapters/sub/1/recurse",
                                "resources/expected_get_chapter1_utf.txt",
                                "application/xml", 200);
+        getAndCompareAsStrings("http://localhost:9080/bookstore/booksubresource/123/chapters/sub/1/recurse2",
+                               "resources/expected_get_chapter1.txt",
+                               "application/xml", "application/xml;charset=iso-8859-1", 200);
     }
     
     @Test
@@ -321,20 +494,18 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     }
     
     @Test
-    public void testGetBookNotFound() throws Exception {
-        
-        getAndCompareAsStrings("http://localhost:9080/bookstore/books/126",
-                               "resources/expected_get_book_notfound.txt",
-                               "application/xml", 500);
+    public void testAddBook() throws Exception {
+        doAddBook("http://localhost:9080/bookstore/books");               
     }
     
     @Test
-    public void testAddBook() throws Exception {
-        String endpointAddress =
-            "http://localhost:9080/bookstore/books";
-
+    public void testAddBookXmlAdapter() throws Exception {
+        doAddBook("http://localhost:9080/bookstore/booksinfo");               
+    }
+    
+    private void doAddBook(String address) throws Exception {
         File input = new File(getClass().getResource("resources/add_book.txt").toURI());         
-        PostMethod post = new PostMethod(endpointAddress);
+        PostMethod post = new PostMethod(address);
         post.setRequestHeader("Content-Type", "application/xml");
         RequestEntity entity = new FileRequestEntity(input, "text/xml; charset=ISO-8859-1");
         post.setRequestEntity(entity);
@@ -350,8 +521,8 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         } finally {
             // Release current connection to the connection pool once you are done
             post.releaseConnection();
-        }               
-    }  
+        }
+    }
     
     @Test
     public void testUpdateBook() throws Exception {
@@ -493,41 +664,22 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         
     @Test
     public void testGetCDs() throws Exception {
-        String endpointAddress =
-            "http://localhost:9080/bookstore/cds"; 
-        URL url = new URL(endpointAddress);
-        URLConnection connect = url.openConnection();
-        connect.addRequestProperty("Accept", "application/xml");
-        InputStream in = connect.getInputStream();
-        assertNotNull(in);           
-
-        InputStream expected124 = getClass().getResourceAsStream("resources/expected_get_cds124.txt");
-        String result = getStringFromInputStream(in);
-        //System.out.println("---" + result);
-        assertTrue(result.indexOf(getStringFromInputStream(expected124)) >= 0);
+        
+        WebClient wc = WebClient.create("http://localhost:9080/bookstore/cds");
+        CDs cds = wc.get(CDs.class);
+        Collection<CD> collection = cds.getCD();
+        assertEquals(2, collection.size());
+        assertTrue(collection.contains(new CD("BICYCLE RACE", 124)));
+        assertTrue(collection.contains(new CD("BOHEMIAN RHAPSODY", 123)));
     }
     
     @Test
     public void testGetCDJSON() throws Exception {
-        String endpointAddress =
-            "http://localhost:9080/bookstore/cd/123"; 
-
-        GetMethod get = new GetMethod(endpointAddress);
-        get.addRequestHeader("Accept" , "application/json");
-
-        HttpClient httpclient = new HttpClient();
         
-        try {
-            int result = httpclient.executeMethod(get);
-            assertEquals(200, result);
-
-            InputStream expected = getClass().getResourceAsStream("resources/expected_get_cdjson.txt");
-            
-            assertEquals(getStringFromInputStream(expected), get.getResponseBodyAsString());
-        } finally {
-            // Release current connection to the connection pool once you are done
-            get.releaseConnection();
-        }  
+        getAndCompareAsStrings("http://localhost:9080/bookstore/cd/123",
+                               "resources/expected_get_cdjson.txt",
+                               "application/json", 200);
+        
     }
     
     @SuppressWarnings("deprecation")
@@ -588,6 +740,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     
     @Test
     public void testGetCDsJSON() throws Exception {
+        
         String endpointAddress =
             "http://localhost:9080/bookstore/cds"; 
 
@@ -614,72 +767,37 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
     
     @Test
     public void testGetCDXML() throws Exception {
-        String endpointAddress =
-            "http://localhost:9080/bookstore/cd/123"; 
-
-        GetMethod get = new GetMethod(endpointAddress);
-        get.addRequestHeader("Accept" , "application/xml");
-
-        HttpClient httpclient = new HttpClient();
         
-        try {
-            int result = httpclient.executeMethod(get);
-            assertEquals(200, result);
-
-            InputStream expected = getClass().getResourceAsStream("resources/expected_get_cd.txt");
-            
-            assertEquals(getStringFromInputStream(expected), get.getResponseBodyAsString());
-        } finally {
-            // Release current connection to the connection pool once you are done
-            get.releaseConnection();
-        }  
+        getAndCompareAsStrings("http://localhost:9080/bookstore/cd/123",
+                               "resources/expected_get_cd.txt",
+                               "application/xml", 200);
     }
     
     
     @Test
     public void testGetCDWithMultiContentTypesXML() throws Exception {
-        String endpointAddress =
-            "http://localhost:9080/bookstore/cdwithmultitypes/123"; 
-
-        GetMethod get = new GetMethod(endpointAddress);
-        get.addRequestHeader("Accept" , "application/xml");
-
-        HttpClient httpclient = new HttpClient();
         
-        try {
-            int result = httpclient.executeMethod(get);
-            assertEquals(200, result);
-
-            InputStream expected = getClass().getResourceAsStream("resources/expected_get_cd.txt");
-            
-            assertEquals(getStringFromInputStream(expected), get.getResponseBodyAsString());
-        } finally {
-            // Release current connection to the connection pool once you are done
-            get.releaseConnection();
-        }  
+        getAndCompareAsStrings("http://localhost:9080/bookstore/cdwithmultitypes/123",
+                               "resources/expected_get_cd.txt",
+                               "application/json;q=0.8,application/xml,*/*", "application/xml", 200);
+    }
+    
+    @Test
+    public void testGetCDWithMultiContentTypesCustomXML() throws Exception {
+        
+        getAndCompareAsStrings("http://localhost:9080/bookstore/cdwithmultitypes/123",
+                               "resources/expected_get_cd.txt",
+                               "application/bar+xml", "application/bar+xml", 200);
     }
     
     @Test
     public void testGetCDWithMultiContentTypesJSON() throws Exception {
-        String endpointAddress =
-            "http://localhost:9080/bookstore/cdwithmultitypes/123"; 
-
-        GetMethod get = new GetMethod(endpointAddress);
-        get.addRequestHeader("Accept" , "application/json");
-
-        HttpClient httpclient = new HttpClient();
-        
-        try {
-            int result = httpclient.executeMethod(get);
-            assertEquals(200, result);
-
-            InputStream expected = getClass().getResourceAsStream("resources/expected_get_cdjson.txt");
-            
-            assertEquals(getStringFromInputStream(expected), get.getResponseBodyAsString());
-        } finally {
-            // Release current connection to the connection pool once you are done
-            get.releaseConnection();
-        }  
+        getAndCompareAsStrings("http://localhost:9080/bookstore/cdwithmultitypes/123",
+                               "resources/expected_get_cdjson.txt",
+                               "application/json", 200);
+        getAndCompareAsStrings("http://localhost:9080/bookstore/cdwithmultitypes/123",
+                               "resources/expected_get_cdjson.txt",
+                               "*/*,application/xml;q=0.9,application/json", "application/json", 200);
     }
     
     @Test
@@ -689,7 +807,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                       "[class org.apache.cxf.systest.jaxrs.BookStore, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
                       + "class org.apache.cxf.systest.jaxrs.Chapter]", 
-                      "text/plain", 200);
+                      "text/plain", "text/plain", 200);
     }
     
     @Test
@@ -699,7 +817,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                       "[class org.apache.cxf.systest.jaxrs.BookStore, " 
                       + "class org.apache.cxf.systest.jaxrs.Book, "
                       + "class org.apache.cxf.systest.jaxrs.Chapter]", 
-                      "text/plain", 200);
+                      "text/plain", "text/plain", 200);
     }
 
     @Test
@@ -709,8 +827,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                           + "/bookstore/booksubresource/123/chapters/sub/1/matched!uris]";
         getAndCompare("http://localhost:9080/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=true", 
-                      expected, 
-                      "text/plain", 200);
+                      expected, "text/plain", "text/plain", 200);
     }
 
     @Test
@@ -722,7 +839,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         getAndCompare("http://localhost:9080/bookstore/"
                       + "booksubresource/123/chapters/sub/1/matched%21uris?decode=false", 
                       expected,
-                      "text/plain", 200);
+                      "text/plain", "text/plain", 200);
     }
     
     private void getAndCompareAsStrings(String address, 
@@ -734,6 +851,21 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
         getAndCompare(address,
                       expected,
                       acceptType,
+                      acceptType,
+                      status);
+    }
+    
+    private void getAndCompareAsStrings(String address, 
+                                        String resourcePath,
+                                        String acceptType,
+                                        String expectedContentType,
+                                        int status) throws Exception {
+        String expected = getStringFromInputStream(
+                              getClass().getResourceAsStream(resourcePath));
+        getAndCompare(address,
+                      expected,
+                      acceptType,
+                      expectedContentType,
                       status);
     }
     
@@ -741,10 +873,22 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
                                String expectedValue,
                                String acceptType,
                                int expectedStatus) throws Exception {
+        getAndCompare(address,
+                      expectedValue,
+                      acceptType,
+                      null,
+                      expectedStatus);
+    }
+    
+    private void getAndCompare(String address, 
+                               String expectedValue,
+                               String acceptType,
+                               String expectedContentType,
+                               int expectedStatus) throws Exception {
         GetMethod get = new GetMethod(address);
         get.setRequestHeader("Accept", acceptType);
-        get.setRequestHeader("Accept-Language", "en,da;q=0.8");
-        get.setRequestHeader("BOOK", "1,2,3");
+        get.setRequestHeader("Accept-Language", "da;q=0.8,en");
+        get.setRequestHeader("Book", "1,2,3");
         HttpClient httpClient = new HttpClient();
         try {
             int result = httpClient.executeMethod(get);
@@ -752,16 +896,27 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
             String content = getStringFromInputStream(get.getResponseBodyAsStream());
             assertEquals("Expected value is wrong", 
                          expectedValue, content);
+            if (expectedStatus == 200) {
+                assertEquals("123", get.getResponseHeader("BookId").getValue());
+                assertNotNull(get.getResponseHeader("Date"));
+            }
+            if (expectedStatus == 405) {
+                assertNotNull(get.getResponseHeader("Allow"));
+            }
+            if (expectedContentType != null) {
+                Header ct = get.getResponseHeader("Content-Type");
+                assertEquals("Wrong type of response", expectedContentType, ct.getValue());
+            }
         } finally {
             get.releaseConnection();
         }
     }
     
     private void getAndCompareStrings(String address, 
-                               String expectedValue1,
-                               String expectedValue2,
+                               String[] expectedValue,
                                String acceptType,
                                int expectedStatus) throws Exception {
+        assertEquals(2, expectedValue.length);
         GetMethod get = new GetMethod(address);
         get.setRequestHeader("Accept", acceptType);
         HttpClient httpClient = new HttpClient();
@@ -770,7 +925,7 @@ public class JAXRSClientServerBookTest extends AbstractBusClientServerTestBase {
             assertEquals(expectedStatus, result);
             String jsonContent = getStringFromInputStream(get.getResponseBodyAsStream());
             assertTrue("Expected value is wrong", 
-                       expectedValue1.equals(jsonContent) || expectedValue2.equals(jsonContent));
+                       expectedValue[0].equals(jsonContent) || expectedValue[1].equals(jsonContent));
         } finally {
             get.releaseConnection();
         }
